function Par_Wave = Init_Waves(Par_Wave)
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 
%
% Init_Waves(Par.Wave)
%
% Project:	"Marine Cybernetics Simulator: MCSim"
%
% Abstract:	Define the harmonic wave components to be used in the simulation by
%			extracting components from a given wave- and direction spectrum.
%			Wave spectrum on ITTC/ISSC PM-form, direction spectrum on cosine form 
% 
% Inputs:	Par_Wave				- Structure containing wave parameters:
%			Par_Wave.spectrum		- Spectrum type: 1 = ITTC, 2 = JONSWAP, 3 = Doubly Peaked
%			Par_Wave.omega_peak		- Peak frequency in spectrum
%			Par_Wave.hs				- Significant wave heigth in sea state
%			Par_Wave.psi_mean		- Mean direction in spectrum
%			Par_Wave.nfreq			- Number of frequency components
%			Par_Wave.ndir			- Number of wave directions
%			Par_Wave.cutoff			- Cutoff frequency of spectrum = cutoff*omega_mean
%			Par_Wave.spread			- Spreading factor for direction spectrum 
%			Par_Wave.plot			- 1 for plotting of spectrum, 0 for not
%			Par_Wave.rand_freq		- 1 for using random frequnencies in each frequency interval
%			Par_Wave.rand_dir		- 1 for using random directions in each direction interval
%			Par.Wave.energylim		- Ignore wave components with Sj/S < energylim, 0 for using all comp.
%			Par.Wave.gamma			-  Gamma value for JONSWAP spectrum, if using this. Default is 3.3
%
% Outputs:	Par_Wave				- Structure containing new wave parameters:
%			Par_Wave.Zeta_a			- Vector of harmonic wave amplitudes
%			Par_Wave.Omega			- Vector of harmonic wave frequencies
%			Par_Wave.Psi_w			- Vector of harmonic wave directions
%			Par_Wave.Phase			- Vector of harmonic wave phases (random)
%	
% Calls:	GNC toolbox Wave spectra: mpierzon, jonswap, torset
%			Dir_Spectrum
%
% Author:	Oyvind Smogeli, Autumn 2001
%           Revision fixingin, Kjell Erik Larsen, Spring 2012
%
% Changes:	OS, Autumn 2002: Changed structure naming for MCSim
%			OS, Spring 2003: Added random frequency and phase pick
%							 Added removal of low energy components
%							 Added GNC Wave spectra
%           KEL, Spring 2012: Seems to be a problem with for looping over
%           the harmonic wave frequencies. Modifying the loop.
%
%
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 


global Glob

% Acceleration of gravity
g = Glob.g;

% FREQUENCY SPECTRUM
 
% Significant wave height
hs = Par_Wave.hs;

% Mean frequency in spectrum
omega_peak = Par_Wave.omega_peak;

% Number of frequency components:
nfreq = Par_Wave.nfreq

% Cutoff frequency
omega_max = Par_Wave.cutoff*omega_peak;

% Frequency step
delta_omega = omega_max/nfreq

% Start frequency
omega_start = delta_omega/2;

% Random frequency usage
rand_freq = Par_Wave.rand_freq;

% Random direction usage
rand_dir = Par_Wave.rand_dir;

% Wave energy lower limit
energylim = Par_Wave.energylim;

% Gamma value for JONSWAP spectrum, if using this. Default is 3.3
gamma = Par_Wave.gamma;

% DIRECTION SPECTRUM

% Mean direction in spectrum
psi_mean = Par_Wave.psi_mean;

% Number of directions 
ndir	= Par_Wave.ndir;		

% Direction step
delta_psi = pi/ndir

% Start direction
psi_start = psi_mean - pi/2 + delta_psi/2;

% Max direction
psi_max = psi_mean + pi/2 - delta_psi/2;


% Wave frequency spectrum, using the GNC toolbox
switch Par_Wave.spectrum 
	
	case 1 % ITTC modified PM spectrum

		[Omega_vec,S_vec,Tz] = mpierson(hs,omega_peak,omega_max,nfreq);

	case 2 % JONSWAP spectrum
		
		%[Omega_vec,S_vec,Tz] = JONSWAP(hs,omega_peak,omega_max,gamma,nfreq);
        Omega_vec=[0:omega_max/Par_Wave.nfreq:omega_max]';
        s_vec=wavespec(7, [hs omega_peak gamma],Omega_vec,0)
        S_vec=[0;s_vec(2:21)];
		
	case 3 % Doubly Peaked spectrum

		[Omega_vec,S_vec] = torset(hs,omega_peak,omega_max,nfreq);

	otherwise
	
		disp('Choose a valid spectrum!!')
		return
		
end

% Close figure from gnc toolbox
close

% Harmonic wave component index
k = 0;

% Frequency index
m = 0;
omega = Omega_vec
% Loop over frequencies
for test = 1:length(Omega_vec)
	
	% Step frequency index
	m = m + 1;
	
    
	% Reset direction index for each frequency
	n = 0;
	
	% Save frequency for plotting
	Omega_plot(m) = omega(m);
	
	s_omega = S_vec(m)
    
	% Loop over directions
	for psi = psi_start:delta_psi:psi_max
		
		% Step direction index 
		n = n + 1;
			
		% Calculate direction spectrum value for each direction component
		s_psi = Dir_Spectrum(Par_Wave.spread, psi, psi_mean)
	
		% Keep wave direction within [-pi, pi]
		psi = rad2pipi(psi);	

		% Save direction for plotting
		Psi_plot(n) = psi;
	
		% Save harmonic wave components if energy is above limit
		
		% Wave component energy
		comp_energy = s_omega*s_psi*delta_omega*delta_psi
		
		% Total sea state energy = m0 ~= (4*Hs)^2, find mean wave energy
		mean_energy = (4*hs)^2/(nfreq*ndir)
		
		if 1 > 0 %comp_energy/mean_energy > 0
		
			% Step component index
			k = k + 1;
			
			% Calculate amplitude
			Zeta_a(k) = sqrt(2*s_omega*s_psi*delta_omega*delta_psi)
			
			% Frequency.
			if rand_freq == 1
				
				% Use randomly chosen frequnencies around the equally spaced ones
				% rand(1) is equally distributed between 0 and 1
				Omega(k) = omega(m) + delta_omega*(rand(1) - 1/2);
				
			else
				
				% Use equally spaced frequencies	
				Omega(k) = omega;
				
			end
			
			% Direction
			if rand_dir == 1
				
				% Use randomly chosen directions around the equally spaced ones
				Psi_w(k) = psi + delta_psi*(rand(1) - 1/2);
				
			else
				
				% Use equally spaced directions
				Psi_w(k) = psi;
				
			end
			
			% Spectrum value (for plotting)
			S_tot(m,n) = s_omega*s_psi;
			
		else % Component discarded due to low energy
			
			% Spectrum value for plotting set to zero
			S_tot(m,n) = 0;
				
		end % Energy test
		
			
	end % direction loop
	
end % frequency loop

% Return error if energy limit set too high
if ~exist('Zeta_a')
	
	disp('Error: Par.Wave.energylim set too high, no wave components generated')
	return
	
end



% Return vectors with wave component values
Par_Wave.Zeta_a	= Zeta_a;				% Amplitude
Par_Wave.Omega	= Omega;				% Frequency
Par_Wave.Psi_w	= Psi_w;				% Direction
Par_Wave.Phase	= rand(1,k)*2*pi;		% Phase

% Plot spectrum 
if Par_Wave.plot

	figure
		
	if ndir > 1
		
		[theta, r] = meshgrid(Psi_plot,Omega_plot);
		[X,Y] = pol2cart(theta, r);
		mesh(Y,X,S_tot);
		xlabel('y')
		ylabel('x')
		zlabel('S [m^2/s]')
		title(sprintf('Wave spectrum, mean direction %0.0f degrees',psi_mean*180/pi))
		set(gca,'XTickLabelMode','manual')
		set(gca,'XTickLabel',{' '})
		set(gca,'YTickLabelMode','manual')
		set(gca,'YTickLabel',{' '})

		
	elseif ndir == 1
		
		plot(Omega, S_tot)
		
	end
	
end % plot

if Par_Wave.realization == 1
	
	% Plot realization of sea surface. Time-consuming!
	
	xdist = 500;
	ydist = 500;
	
	xpoints = 50;
	ypoints = 50;
	
	t = 0;
	
	az = -50;
	el = 70;
	
	%----------------------------------------------------------------
	
	g = Glob.g;
	
	Zeta = zeros(xpoints+1, ypoints+1);
	Z_temp = Zeta;
	
	dx = xdist/xpoints;
	dy = ydist/ypoints;
	
	xvec = 0:dx:xdist;
	yvec = 0:dy:ydist;
	
	nwaves = length(Par_Wave.Zeta_a);
	
	hh = waitbar(0,'Running sea state realization. Set option in Parameters.m');
	
	for n = 1:nwaves
		
		% Particulars for one harmonic wave 
		zeta_a = Par_Wave.Zeta_a(n);
		psi_w = Par_Wave.Psi_w(n);
		omega_w = Par_Wave.Omega(n);
		phase_w = Par_Wave.Phase(n);
		
		xc = 0;
		
		for x = xvec
			
			xc = xc + 1;
			
			yc = 0;
			
			for y = yvec
				
				yc = yc + 1;
				
				x_rel = rotate_yaw_T(psi_w)*[x, y, 0]';
				x_rel = x_rel(1);
				
				% Wave number k from deepwater dispersion relation
				k = omega_w^2/g;
				
				% Phase due to x_rel, negative because it gives lag:
				phase_pos = -x_rel*k; % [rad] (= -x_rel/lambda)
				
				% Wave elevation
				Z_temp(xc,yc) = zeta_a*sin(omega_w*t + phase_w + phase_pos);
				
			end %y
			
		end %x
		
		waitbar(n/nwaves, hh);
		
		Zeta = Zeta + Z_temp;
		
	end % waves
	
	close(hh)
	
	% Test if hs is correct
	hs_sim=4*sqrt(mean(mean(Zeta.^2)));

	disp(' ')
	disp('***********************************************************')
	disp('Sea state total energy test:')
	disp(['hs input: ', num2str(Par_Wave.hs)])
	disp(['hs sim:   ', num2str(hs_sim)])
	disp('***********************************************************')
	
	figure
	mesh(yvec,xvec,Zeta)
	xlabel('y')
	ylabel('x')
	zlabel('z')
	title('Wave realization')
	view(az,el)
	
	
end